@@ -1,7 +1,7 @@
 /* This example demonstrates how to retreive SSL/TLS server credentials.
  */
-#include "libepc/tls.h"
-#include "libepc-ui/entropy-window.h"
+#include <libepc/tls.h>
+#include <libepc-ui/progress-window.h>
 #include <gtk/gtk.h>
 
 int
@@ -19,19 +19,18 @@ main (int   argc,
    */
   gnutls_global_init ();
   g_thread_init (NULL);
-  gdk_threads_init ();
   gtk_init (&argc, &argv);
 
   /* Show a progress window when generating new keys.
    */
-  epc_entropy_window_install ();
+  epc_progress_window_install (NULL);
 
   /* Retreive and display of server credentials.
    */
   if (epc_tls_get_server_credentials (hostname, &crtfile, &keyfile, &error))
     {
       g_print ("private key file: %s\n", keyfile);
-      
+
       if (!g_file_get_contents (keyfile, &contents, NULL, &error))
         goto out;
 
