@@ -28,6 +28,10 @@
 #include <glibconfig.h>
 #include <glib/gi18n-lib.h>
 
+#if GLIB_CHECK_VERSION(2,15,0)
+#include <gio/gcontenttype.h>
+#endif
+
 #include <libsoup/soup-address.h>
 #include <libsoup/soup-server.h>
 #include <libsoup/soup-server-auth.h>
@@ -47,7 +51,7 @@
  * the #EpcAuthHandler registered for the key in question, if any. Information about
  * that process is stored in the #EpcAuthContext structure.
  *
- * To register an authentication handler call #epc_publisher_set_auth_handler:
+ * To register an authentication handler call epc_publisher_set_auth_handler():
  *
  * <example id="register-auth-handler">
  *  <title>Register an authentication handler</title>
@@ -58,7 +62,7 @@
  * </example>
  *
  * To verify that the user password provided password matches
- * the expected one use #epc_auth_context_check_password:
+ * the expected one use epc_auth_context_check_password():
  *
  * <example id="check-password">
  *  <title>Verify a password</title>
@@ -113,12 +117,12 @@
  * a %NULL value essentially you say "this information does not exist". So
  * publishing %NULL values is not different from not publishing any value at
  * all or rejected access to some values. Without explicitly inspecting the
- * details for not receiving a value, a consumer calling #epc_consumer_lookup
+ * details for not receiving a value, a consumer calling epc_consumer_lookup()
  * has no chance to distinguish between the cases "never published", "network
  * problem", "authorization rejected", "no meaningful value available".
  *
  * So if  feel like publishing a %NULL value, just remove the key in question
- * from the #EpcPublisher by calling #epc_publisher_remove. When using a
+ * from the #EpcPublisher by calling epc_publisher_remove(). When using a
  * custom #EpcContentsHandler an alternate approach is returning %NULL from
  * that handler. In that case the #EpcPublisher will behave exactly the same,
  * as if the value has been removed.
@@ -131,9 +135,11 @@ enum
 {
   PROP_NONE,
   PROP_PROTOCOL,
+  PROP_APPLICATION,
   PROP_SERVICE_NAME,
   PROP_SERVICE_DOMAIN,
-  PROP_APPLICATION,
+  PROP_SERVICE_COOKIE,
+  PROP_COLLISION_HANDLING,
 
   PROP_AUTH_FLAGS,
   PROP_CONTENTS_PATH,
@@ -146,7 +152,7 @@ enum
  *
  * This data structure describes a pending authentication request
  * which shall be verified by an #EpcAuthHandler installed by
- * #epc_publisher_set_auth_handler.
+ * epc_publisher_set_auth_handler().
  *
  * <note><para>
  *  There is no way to retrieve the password from the #EpcAuthContext, as
@@ -203,11 +209,13 @@ struct _EpcPublisherPrivate
   GHashTable            *clients;
 
   EpcProtocol            protocol;
+  gchar                 *application;
   gchar                 *service_name;
   gchar                 *service_domain;
-  gchar                 *application;
+  gchar                 *service_cookie;
 
   EpcAuthFlags           auth_flags;
+  EpcCollisionHandling   collisions;
   gchar                 *contents_path;
   gchar                 *certificate_file;
   gchar                 *private_key_file;
@@ -300,9 +308,17 @@ epc_publisher_handle_file (EpcPublisher *publisher G_GNUC_UNUSED,
   gchar *data = NULL;
   gsize length = 0;
 
-  /* TODO: use gio to determinate mime-type */
   if (g_file_get_contents (filename, &data, &length, NULL))
-    contents = epc_contents_new (NULL, data, length, g_free);
+    {
+      gchar *type = NULL;
+
+#if GLIB_CHECK_VERSION(2,15,0)
+      type = g_content_type_guess (filename, (gpointer) data, length, NULL);
+#endif
+      contents = epc_contents_new (type, data, length, g_free);
+
+      g_free (type);
+    }
 
   return contents;
 }
@@ -945,6 +961,10 @@ epc_publisher_create_server (EpcPublisher  *self,
 
   self->priv->dispatcher = epc_dispatcher_new (epc_publisher_compute_name (self));
 
+  if (self->priv->service_cookie)
+    epc_dispatcher_set_cookie (self->priv->dispatcher, self->priv->service_cookie);
+  epc_dispatcher_set_collision_handling (self->priv->dispatcher, self->priv->collisions);
+
   if (!epc_dispatcher_run (self->priv->dispatcher, error))
     return FALSE;
 
@@ -1040,6 +1060,42 @@ epc_publisher_real_set_auth_flags (EpcPublisher *self,
     epc_publisher_install_handlers (self);
 }
 
+/**
+ * epc_publisher_set_service_cookie:
+ * @publisher: a #EpcPublisher
+ * @cookie: the new service identifier, or %NULL
+ *
+ * Changes the unique identifier of the service.
+ * See #EpcPublisher:service-cookie for details.
+ *
+ * Since: 0.3.1
+ */
+void
+epc_publisher_set_service_cookie (EpcPublisher *self,
+                                  const gchar  *cookie)
+{
+  g_return_if_fail (EPC_IS_PUBLISHER (self));
+  g_object_set (self, "service-cookie", cookie, NULL);
+}
+
+/**
+ * epc_publisher_set_collision_handling:
+ * @publisher: a #EpcPublisher
+ * @method: the new strategy
+ *
+ * Changes the collision handling strategy the publisher uses.
+ * See #EpcPublisher:collision-handling for details.
+ *
+ * Since: 0.3.1
+ */
+void
+epc_publisher_set_collision_handling (EpcPublisher         *self,
+                                      EpcCollisionHandling  method)
+{
+  g_return_if_fail (EPC_IS_PUBLISHER (self));
+  g_object_set (self, "collision-handling", method, NULL);
+}
+
 static void
 epc_publisher_real_set_contents_path (EpcPublisher *self,
                                       const GValue *value)
@@ -1080,6 +1136,13 @@ epc_publisher_set_property (GObject      *object,
         self->priv->protocol = g_value_get_enum (value);
         break;
 
+      case PROP_APPLICATION:
+        g_return_if_fail (!epc_publisher_is_server_created (self));
+
+        g_free (self->priv->application);
+        self->priv->application = g_value_dup_string (value);
+        break;
+
       case PROP_SERVICE_NAME:
         epc_publisher_real_set_service_name (self, value);
         break;
@@ -1091,11 +1154,22 @@ epc_publisher_set_property (GObject      *object,
         self->priv->service_domain = g_value_dup_string (value);
         break;
 
-      case PROP_APPLICATION:
-        g_return_if_fail (!epc_publisher_is_server_created (self));
+      case PROP_SERVICE_COOKIE:
+        g_free (self->priv->service_cookie);
+        self->priv->service_cookie = g_value_dup_string (value);
+
+        if (self->priv->dispatcher)
+          epc_dispatcher_set_cookie (self->priv->dispatcher,
+                                     self->priv->service_cookie);
+        break;
+
+      case PROP_COLLISION_HANDLING:
+        self->priv->collisions = g_value_get_enum (value);
+
+        if (self->priv->dispatcher)
+          epc_dispatcher_set_collision_handling (self->priv->dispatcher,
+                                                 self->priv->collisions);
 
-        g_free (self->priv->application);
-        self->priv->application = g_value_dup_string (value);
         break;
 
       case PROP_CONTENTS_PATH:
@@ -1140,6 +1214,10 @@ epc_publisher_get_property (GObject    *object,
         g_value_set_enum (value, self->priv->protocol);
         break;
 
+      case PROP_APPLICATION:
+        g_value_set_string (value, self->priv->application);
+        break;
+
       case PROP_SERVICE_NAME:
         g_value_set_string (value, self->priv->service_name);
         break;
@@ -1148,8 +1226,12 @@ epc_publisher_get_property (GObject    *object,
         g_value_set_string (value, self->priv->service_domain);
         break;
 
-      case PROP_APPLICATION:
-        g_value_set_string (value, self->priv->application);
+      case PROP_SERVICE_COOKIE:
+        g_value_set_string (value, self->priv->service_cookie);
+        break;
+
+      case PROP_COLLISION_HANDLING:
+        g_value_set_enum (value, self->priv->collisions);
         break;
 
       case PROP_CONTENTS_PATH:
@@ -1240,6 +1322,14 @@ epc_publisher_class_init (EpcPublisherClass *cls)
                                                       G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
                                                       G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (oclass, PROP_APPLICATION,
+                                   g_param_spec_string ("application", "Application",
+                                                        "Program name for deriving the service type",
+                                                        NULL,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_BLURB));
+
   g_object_class_install_property (oclass, PROP_SERVICE_NAME,
                                    g_param_spec_string ("service-name", "Service Name",
                                                         "User friendly name for the service",
@@ -1256,14 +1346,41 @@ epc_publisher_class_init (EpcPublisherClass *cls)
                                                         G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
 
-  g_object_class_install_property (oclass, PROP_APPLICATION,
-                                   g_param_spec_string ("application", "Application",
-                                                        "Program name for deriving the service type",
+  /**
+   * EpcPublisher:service-cookie:
+   *
+   * Unique identifier of the service. This cookie is used for implementing
+   * #EPC_COLLISIONS_UNIQUE_SERVICE, and usually is a UUID or the MD5/SHA1/...
+   * checksum of a central document. When passing %NULL, but using the
+   * #EPC_COLLISIONS_UNIQUE_SERVICE strategy a time based UUID is
+   * generated and used as service identifier.
+   *
+   * Since: 0.3.1
+   */
+  g_object_class_install_property (oclass, PROP_SERVICE_COOKIE,
+                                   g_param_spec_string ("service-cookie", "Service Cookie",
+                                                        "Unique identifier of the service",
                                                         NULL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
                                                         G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
 
+  /**
+   * EpcPublisher:collision-handling:
+   *
+   * The collision handling strategy the publisher uses.
+   *
+   * Since: 0.3.1
+   */
+  g_object_class_install_property (oclass, PROP_COLLISION_HANDLING,
+                                   g_param_spec_enum ("collision-handling", "Collision Handling",
+                                                      "The collision handling strategy to use",
+                                                      EPC_TYPE_COLLISION_HANDLING,
+                                                      EPC_COLLISIONS_CHANGE_NAME,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
+                                                      G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
+                                                      G_PARAM_STATIC_BLURB));
+
   g_object_class_install_property (oclass, PROP_CONTENTS_PATH,
                                    g_param_spec_string ("contents-path", "Contents Path",
                                                         "The built-in server path for publishing resources",
@@ -1309,8 +1426,8 @@ epc_publisher_class_init (EpcPublisherClass *cls)
  * Creates a new #EpcPublisher object. The publisher announces its service
  * per DNS-SD to the DNS domain specified by @domain, using @name as service
  * name. The service type is derived from @application. When %NULL is passed
- * for @application the value returned by #g_get_prgname is used. See
- * #epc_service_type_new for details.
+ * for @application the value returned by g_get_prgname() is used. See
+ * epc_service_type_new() for details.
  *
  * Returns: The newly created #EpcPublisher object.
  */
@@ -1553,9 +1670,9 @@ epc_publisher_remove (EpcPublisher *self,
  * @publisher: a #EcpPublisher
  * @key: the key for addressing contents
  *
- * Looks up the user_data passed to #epc_publisher_add_handler for @key.
+ * Looks up the user_data passed to epc_publisher_add_handler() for @key.
  * Returns %NULL if the specified @key doesn't exist or wasn't published
- * with #epc_publisher_add_handler.
+ * with epc_publisher_add_handler().
  *
  * This function allows to use the publisher as local key/value store,
  * which is useful for instance to prevent accidental key collisions.
@@ -1649,7 +1766,7 @@ epc_publisher_list_cb (gpointer key,
  *  g_list_free (keys);
  * </programlisting>
  *
- * See also #epc_consumer_list for builtin listing capabilities.
+ * See also epc_consumer_list() for builtin listing capabilities.
  *
  * Returns: A newly allocated list of keys, or %NULL when an error occurred.
  */
@@ -1701,10 +1818,10 @@ epc_publisher_list (EpcPublisher *self,
  *
  * <note><para>
  *  This should be called after adding the resource identified by @key,
- *  not before. For instance, after calling #epc_publisher_add.
+ *  not before. For instance, after calling epc_publisher_add().
  * </para></note>
  *
- * See also #epc_publisher_set_auth_flags.
+ * See also epc_publisher_set_auth_flags().
  */
 void
 epc_publisher_set_auth_handler (EpcPublisher   *self,
@@ -1752,7 +1869,7 @@ epc_publisher_set_auth_handler (EpcPublisher   *self,
  *
  * <note><para>
  *  This should be called after adding the resource identified by @key,
- *  not before. For instance, after calling #epc_publisher_add.
+ *  not before. For instance, after calling epc_publisher_add().
  * </para></note>
  */
 void
@@ -1863,7 +1980,7 @@ epc_publisher_set_contents_path (EpcPublisher *self,
  * @flags: new authentication settings
  *
  * Changes the authentication settings the publisher uses 
- * when #epc_publisher_set_auth_handler is used.
+ * when epc_publisher_set_auth_handler() is used.
  * See #EpcPublisher:auth-flags for details.
  */
 void
@@ -1988,26 +2105,61 @@ epc_publisher_get_auth_flags (EpcPublisher *self)
   return self->priv->auth_flags;
 }
 
+/**
+ * epc_publisher_get_service_cookie:
+ * @publisher: a #EpcPublisher
+ *
+ * Queries the unique identifier of the service.
+ * See #EpcPublisher:service-cookie for details.
+ *
+ * Returns: The unique identifier of the service, or %NULL on error.
+ * Since: 0.3.1
+ */
+G_CONST_RETURN gchar*
+epc_publisher_get_service_cookie (EpcPublisher *self)
+{
+  g_return_val_if_fail (EPC_IS_PUBLISHER (self), NULL);
+  return self->priv->service_cookie;
+}
+
+/**
+ * epc_publisher_get_collision_handling:
+ * @publisher: a #EpcPublisher
+ *
+ * Queries the collision handling strategy the publisher uses.
+ * See #EpcPublisher:collision-handling for details.
+ *
+ * Returns: The publisher's collision handling strategy,
+ * or #EPC_COLLISIONS_IGNORE on error.
+ * Since: 0.3.1
+ */
+EpcCollisionHandling
+epc_publisher_get_collision_handling (EpcPublisher *self)
+{
+  g_return_val_if_fail (EPC_IS_PUBLISHER (self), EPC_COLLISIONS_IGNORE);
+  return self->priv->collisions;
+}
+
 /**
  * epc_publisher_run:
  * @publisher: a #EpcPublisher
  * @error: return location for a #GError, or %NULL
  *
  * Starts the server component of the #EpcPublisher and blocks until it is
- * shutdown using #epc_publisher_quit. If the server could not be started, the
+ * shutdown using epc_publisher_quit(). If the server could not be started, the
  * function returns %FALSE and sets @error. The error domain is
  * #EPC_AVAHI_ERROR. Possible error codes are those of the
  * <citetitle>Avahi</citetitle> library.
  *
  * When starting the publisher in HTTPS mode for the first time self-signed
  * keys must be generated. Generating secure keys needs some time,
- * so it is recommended to call #epc_progress_window_install, or
- * #epc_shell_set_progress_hooks to provide visual feedback during that
+ * so it is recommended to call epc_progress_window_install(), or
+ * epc_shell_set_progress_hooks() to provide visual feedback during that
  * operation. Key generation takes place in a separate background thread and
  * the calling thread waits in a GMainLoop. Therefore the UI can remain
  * responsive when generating keys.
  *
- * To start the server without blocking call #epc_publisher_run_async.
+ * To start the server without blocking call epc_publisher_run_async().
  *
  * Returns: %TRUE when the publisher was successfully started,
  * %FALSE if an error occurred.
@@ -2044,8 +2196,8 @@ epc_publisher_run (EpcPublisher  *self,
  * error domain is #EPC_AVAHI_ERROR. Possible error codes are those of the
  * <citetitle>Avahi</citetitle> library.
  *
- * To stop the server component call #epc_publisher_quit.
- * See #epc_publisher_run for additional information.
+ * To stop the server component call epc_publisher_quit().
+ * See epc_publisher_run() for additional information.
  *
  * Returns: %TRUE when the publisher was successfully started,
  * %FALSE if an error occurred.
@@ -2073,14 +2225,17 @@ epc_publisher_run_async (EpcPublisher  *self,
 static void
 epc_publisher_disconnect_idle_cb (gpointer key,
                                   gpointer value,
-                                  gpointer data G_GNUC_UNUSED)
+                                  gpointer data)
 {
+  SoupSocket *socket = key;
+  GSList **clients = data;
+
   if (1 >= GPOINTER_TO_INT (value))
     {
       if (EPC_DEBUG_LEVEL (1))
-        epc_publisher_trace_client (G_STRFUNC, "idle client", key);
+        epc_publisher_trace_client (G_STRFUNC, "idle client", socket);
 
-      soup_socket_disconnect (key);
+      *clients = g_slist_prepend (*clients, socket);
     }
 }
 
@@ -2089,7 +2244,7 @@ epc_publisher_disconnect_idle_cb (gpointer key,
  * @publisher: a #EpcPublisher
  *
  * Stops the server component of the #EpcPublisher started with
- * #epc_publisher_run or #epc_publisher_run_async. The functions
+ * epc_publisher_run() or #epc_publisher_run_async. The functions
  * returns %TRUE when the built-in server was running and had to
  * be stopped. If the server wasn't running the function returns
  * %FALSE.
@@ -2099,6 +2254,7 @@ epc_publisher_disconnect_idle_cb (gpointer key,
 gboolean
 epc_publisher_quit (EpcPublisher *self)
 {
+  GSList *idle_clients = NULL;
   gboolean was_running;
 
   g_return_val_if_fail (EPC_IS_PUBLISHER (self), FALSE);
@@ -2113,7 +2269,10 @@ epc_publisher_quit (EpcPublisher *self)
   if (self->priv->clients)
     g_hash_table_foreach (self->priv->clients,
                           epc_publisher_disconnect_idle_cb,
-                          self);
+                          &idle_clients);
+
+  g_slist_foreach (idle_clients, (GFunc) soup_socket_disconnect, NULL);
+  g_slist_free (idle_clients);
 
   g_static_rec_mutex_unlock (&epc_publisher_lock);
 
@@ -2189,7 +2348,7 @@ epc_utf8_strtitle (const gchar *str,
  * Expands all known placeholders in @name. Supported placeholders are:
  *
  * <itemizedlist>
- *  <listitem>%%a: the program name as returned by #g_get_application_name</listitem>
+ *  <listitem>%%a: the program name as returned by g_get_application_name()</listitem>
  *  <listitem>%%h: the machine's host name in title case</listitem>
  *  <listitem>%%u: the user's login name in title case</listitem>
  *  <listitem>%%U: the user's real name</listitem>
