@@ -373,7 +373,6 @@ epc_service_monitor_class_init (EpcServiceMonitorClass *cls)
                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
                                                          G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
-  
 
   /**
    * EpcServiceMonitor::service-found:
@@ -422,54 +421,82 @@ epc_service_monitor_class_init (EpcServiceMonitorClass *cls)
 }
 
 /**
- * epc_service_monitor_new_for_types:
+ * epc_service_monitor_new_for_types_strv:
  * @domain: the DNS domain to monitor, or %NULL
- * @first_service_type: the first service type to monitor
- * @...: a %NULL terminated list of additional service types
+ * @types: a %NULL terminated list of service types to monitor
  *
  * Creates a new service monitor watching the specified @domain for the
  * service-types listed. Passing %NULL for @domain monitors the local network.
  * Passing an empty service list requests monitoring of all service-types
- * supported by the library (see #epc_service_type_list_supported).
+ * supported by the library (see epc_service_type_list_supported()).
+ *
+ * See also: epc_service_monitor_new_for_types()
  *
  * Returns: The newly created service monitor.
  */
 EpcServiceMonitor*
+epc_service_monitor_new_for_types_strv (const gchar  *domain,
+                                        gchar       **types)
+{
+  g_return_val_if_fail (NULL != types, NULL);
+
+  return g_object_new (EPC_TYPE_SERVICE_MONITOR,
+                       "service-types", types,
+                       "domain", domain,
+                       NULL);
+}
+
+/**
+ * epc_service_monitor_new_for_types:
+ * @domain: the DNS domain to monitor, or %NULL
+ * @...: a %NULL terminated list of service types to monitor
+ *
+ * Creates a new service monitor watching the specified @domain for the
+ * service-types listed. Passing %NULL for @domain monitors the local network.
+ * Passing an empty service list requests monitoring of all service-types
+ * supported by the library (see epc_service_type_list_supported()).
+ *
+ * See also: epc_service_monitor_new_for_types_strv()
+ *
+ * Returns: The newly created service monitor.
+ * Since: 0.3.1
+ */
+EpcServiceMonitor*
 epc_service_monitor_new_for_types (const gchar *domain,
-                                   const gchar *first_service_type,
                                                 ...)
 {
-  gchar **service_types = NULL;
+  EpcServiceMonitor *self = NULL;
+  gchar **types = NULL;
   va_list args;
   gint i;
 
   for (i = 0; i < 2; ++i)
     {
-      const gchar *type = first_service_type;
+      const gchar *type;
       gint tail = 0;
 
-      va_start (args, first_service_type);
+      va_start (args, domain);
 
-      while (NULL != type)
+      while (NULL != (type = va_arg (args, const gchar*)))
         {
-          type = va_arg (args, const gchar*);
-
-          if (service_types)
-            service_types[tail] = g_strdup (type);
+          if (types)
+            types[tail] = g_strdup (type);
 
           tail += 1;
         }
 
       va_end (args);
 
-      if (NULL == service_types)
-        service_types = g_new0 (gchar*, tail + 1);
+      if (NULL == types)
+        types = g_new0 (gchar*, tail + 1);
     }
 
-  return g_object_new (EPC_TYPE_SERVICE_MONITOR,
-                       "service-types", service_types,
-                       "domain", domain,
-                       NULL);
+  self = g_object_new (EPC_TYPE_SERVICE_MONITOR,
+                       "service-types", types,
+                       "domain", domain, NULL);
+
+  g_strfreev (types);
+  return self;
 }
 
 /**
@@ -484,7 +511,7 @@ epc_service_monitor_new_for_types (const gchar *domain,
  * @application lists all libepc based applications. Passing %NULL for @domain
  * monitors the local network. Passing an empty protocol list requests
  * monitoring of all service-types supported by the library (see
- * #epc_service_type_list_supported).
+ * epc_service_type_list_supported()).
  *
  * Returns: The newly created service monitor.
  */
@@ -494,7 +521,8 @@ epc_service_monitor_new (const gchar *application,
                          EpcProtocol  first_protocol,
                                       ...)
 {
-  gchar **service_types = NULL;
+  EpcServiceMonitor* self = NULL;
+  gchar **types = NULL;
   va_list args;
   gint i;
 
@@ -507,25 +535,27 @@ epc_service_monitor_new (const gchar *application,
 
       while (((gint) protocol) > EPC_PROTOCOL_UNKNOWN)
         {
+          if (types)
+            types[tail] = epc_service_type_new (protocol, application);
+
           protocol = va_arg (args, EpcProtocol);
-
-          if (service_types)
-            service_types[tail] = epc_service_type_new (protocol, application);
-
           tail += 1;
         }
 
       va_end (args);
 
-      if (NULL == service_types)
-        service_types = g_new0 (gchar*, tail + 1);
+      if (NULL == types)
+        types = g_new0 (gchar*, tail + 1);
     }
 
-  return g_object_new (EPC_TYPE_SERVICE_MONITOR,
-                       "service-types", service_types,
+  self = g_object_new (EPC_TYPE_SERVICE_MONITOR,
                        "application", application,
+                       "service-types", types,
                        "domain", domain,
                        NULL);
+
+  g_strfreev (types);
+  return self;
 }
 
 /**
